@@ -160,6 +160,26 @@ static const size_t headerMaxbytes = (256*1024*1024);
 static int dataLength(rpm_tagtype_t type, rpm_constdata_t p, rpm_count_t count,
 			 int onDisk, rpm_constdata_t pend);
 
+/* Check tag type matches our definition */
+static int hdrchkTagType(rpm_tag_t tag, rpm_tagtype_t type)
+{
+    rpmTagType t = rpmTagGetTagType(tag);
+    if (t == type)
+	return 0;
+
+    /* Permit unknown tags for forward compatibility */
+    if (t == RPM_NULL_TYPE)
+	return 0;
+
+    /* Some string tags harmlessly disagree on the exact type */
+    if (rpmTagGetClass(tag) == RPM_STRING_CLASS &&
+	    (rpmTagTypeGetClass(type) == RPM_STRING_CLASS))
+	return 0;
+
+    /* Known tag with mismatching type, bad bad bad. */
+    return 1;
+}
+
 #ifndef htonll
 /* Convert a 64bit value to network byte order. */
 RPM_GNUC_CONST
@@ -253,6 +273,9 @@ static rpmRC hdrblobVerifyInfo(hdrblob blob, char **emsg)
     const char *ds = (const char *) blob->dataStart;
     int32_t il = (blob->regionTag) ? blob->il-1 : blob->il;
     entryInfo pe = (blob->regionTag) ? blob->pe+1 : blob->pe;
+    /* Can't typecheck signature header tags, sigh */
+    int typechk = (blob->regionTag == RPMTAG_HEADERIMMUTABLE ||
+		   blob->regionTag == RPMTAG_HEADERIMAGE);
 
     for (i = 0; i < il; i++) {
 	ei2h(&pe[i], &info);
@@ -269,6 +292,8 @@ static rpmRC hdrblobVerifyInfo(hdrblob blob, char **emsg)
 	    goto err;
 	if (hdrchkRange(blob->dl, info.offset))
 	    goto err;
+	if (typechk && hdrchkTagType(info.tag, info.type))
+	    goto err;
 
 	/* Verify the data actually fits */
 	len = dataLength(info.type, ds + info.offset,
@@ -497,7 +522,8 @@ static int regionSwab(indexEntry entry, int il, int dl,
 	if (dataEnd && (unsigned char *)ie.data >= dataEnd)
 	    return -1;
 
-	if (fast && il > 1) {
+	/* The offset optimization is only relevant for string types */
+	if (fast && il > 1 && typeSizes[ie.info.type] == -1) {
 	    ie.length = ntohl(pe[1].offset) - ie.info.offset;
 	} else {
 	    ie.length = dataLength(ie.info.type, ie.data, ie.info.count,
@@ -1104,7 +1130,6 @@ static int copyTdEntry(const indexEntry entry, rpmtd td, headerGetFlags flags)
 	    /* XXX 1 on success. */
 	    rc = (rc < 0) ? 0 : 1;
 	} else {
-	    count = entry->length;
 	    td->data = (!minMem
 		? memcpy(xmalloc(count), entry->data, count)
 		: entry->data);
@@ -1817,6 +1842,9 @@ static rpmRC hdrblobVerifyRegion(rpmTagVal regionTag, int exact_size,
     ei2h(&trailer, &einfo);
     /* Trailer offset is negative and has a special meaning */
     einfo.offset = -einfo.offset;
+    /* Some old packages have HEADERIMAGE in signature region trailer, fix up */
+    if (regionTag == RPMTAG_HEADERSIGNATURES && einfo.tag == RPMTAG_HEADERIMAGE)
+	einfo.tag = RPMTAG_HEADERSIGNATURES;
     if (!(einfo.tag == regionTag &&
 	  einfo.type == REGION_TAG_TYPE && einfo.count == REGION_TAG_COUNT))
     {
@@ -1850,6 +1878,21 @@ exit:
     return rc;
 }
 
+hdrblob hdrblobCreate(void)
+{
+    hdrblob blob = xcalloc(1, sizeof(*blob));
+    return blob;
+}
+
+hdrblob hdrblobFree(hdrblob blob)
+{
+    if (blob) {
+	free(blob->ei);
+	free(blob);
+    }
+    return NULL;
+}
+
 rpmRC hdrblobRead(FD_t fd, int magic, int exact_size, rpmTagVal regionTag, hdrblob blob, char **emsg)
 {
     int32_t block[4];
@@ -1976,7 +2019,7 @@ rpmRC hdrblobGet(hdrblob blob, uint32_t tag, rpmtd td)
     memset(&einfo, 0, sizeof(einfo));
     rpmtdReset(td);
 
-    for (int i = 1; i < blob->il; i++, pe++) {
+    for (int i = 0; i < blob->il; i++, pe++) {
 	if (pe->tag != ntag)
 	    continue;
 	ei2h(pe, &einfo);
