@@ -20,11 +20,23 @@
 
 #include "debug.h"
 
+typedef struct pkgdata_s *pkgdatap;
+
+typedef void (*hdrvsmsg)(struct rpmsinfo_s *sinfo, pkgdatap pkgdata, const char *msg);
+
+struct pkgdata_s {
+    hdrvsmsg msgfunc;
+    const char *fn;
+    char *msg;
+    rpmRC rc;
+};
+
 /** \ingroup header
  * Translate and merge legacy signature tags into header.
  * @param h		header (dest)
  * @param sigh		signature header (src)
  */
+static
 void headerMergeLegacySigs(Header h, Header sigh)
 {
     HeaderIterator hi;
@@ -137,15 +149,32 @@ exit:
     return seen;
 }
 
-static rpmRC handleHdrVS(struct rpmsinfo_s *sinfo, rpmRC rc, const char *msg, void *cbdata)
+static int handleHdrVS(struct rpmsinfo_s *sinfo, void *cbdata)
 {
-    char **buf  = cbdata;
-    if (buf) {
-	char *vsmsg = rpmsinfoMsg(sinfo, rc, msg);
-	*buf = rstrscat(buf, "\n", vsmsg, NULL);
+    struct pkgdata_s *pkgdata = cbdata;
+
+    if (pkgdata->msgfunc) {
+	char *vsmsg = rpmsinfoMsg(sinfo);
+	pkgdata->msgfunc(sinfo, pkgdata, vsmsg);
 	free(vsmsg);
     }
-    return rc;
+
+    /* Remember actual return code, but don't override a previous failure */
+    if (sinfo->rc && pkgdata->rc != RPMRC_FAIL)
+	pkgdata->rc = sinfo->rc;
+
+    /* Preserve traditional behavior for now: only failure prevents read */
+    if (sinfo->rc != RPMRC_FAIL)
+	sinfo->rc = RPMRC_OK;
+
+    return 1;
+}
+
+
+static void appendhdrmsg(struct rpmsinfo_s *sinfo, struct pkgdata_s *pkgdata,
+			const char *msg)
+{
+    pkgdata->msg = rstrscat(&pkgdata->msg, "\n", msg, NULL);
 }
 
 static void updateHdrDigests(rpmDigestBundle bundle, struct hdrblob_s *blob)
@@ -164,23 +193,37 @@ rpmRC headerCheck(rpmts ts, const void * uh, size_t uc, char ** msg)
     rpmVSFlags vsflags = rpmtsVSFlags(ts) | RPMVSF_NEEDPAYLOAD;
     rpmKeyring keyring = rpmtsGetKeyring(ts, 1);
     struct hdrblob_s blob;
+    struct pkgdata_s pkgdata = {
+	.msgfunc = appendhdrmsg,
+	.fn = NULL,
+	.msg = NULL,
+	.rc = RPMRC_OK,
+    };
 
     if (hdrblobInit(uh, uc, 0, 0, &blob, msg) == RPMRC_OK) {
-	struct rpmvs_s *vs = rpmvsCreate(&blob, vsflags);
+	struct rpmvs_s *vs = rpmvsCreate(0, vsflags, keyring);
 	rpmDigestBundle bundle = rpmDigestBundleNew();
 
 	rpmswEnter(rpmtsOp(ts, RPMTS_OP_DIGEST), 0);
 
-	rpmvsInitDigests(vs, RPMSIG_HEADER, bundle);
+	rpmvsInit(vs, &blob, bundle);
+	rpmvsInitRange(vs, RPMSIG_HEADER);
 	updateHdrDigests(bundle, &blob);
+	rpmvsFiniRange(vs, RPMSIG_HEADER);
 
-	rc = rpmvsVerifyItems(vs, RPMSIG_HEADER, bundle, keyring,
-				handleHdrVS, msg);
+	rpmvsVerify(vs, RPMSIG_VERIFIABLE_TYPE, handleHdrVS, &pkgdata);
 
 	rpmswExit(rpmtsOp(ts, RPMTS_OP_DIGEST), uc);
 
-	if (rc == RPMRC_OK && msg != NULL && *msg == NULL)
-	    rasprintf(msg, "Header sanity check: OK");
+	rc = pkgdata.rc;
+
+	if (rc == RPMRC_OK && pkgdata.msg == NULL)
+	    pkgdata.msg = xstrdup("Header sanity check: OK");
+
+	if (msg)
+	    *msg = pkgdata.msg;
+	else
+	    free(pkgdata.msg);
 
 	rpmDigestBundleFree(bundle);
 	rpmvsFree(vs);
@@ -223,7 +266,8 @@ exit:
     return rc;
 }
 
-void applyRetrofits(Header h, int leadtype)
+static
+void applyRetrofits(Header h)
 {
     /*
      * Make sure that either RPMTAG_SOURCERPM or RPMTAG_SOURCEPACKAGE
@@ -258,17 +302,11 @@ void applyRetrofits(Header h, int leadtype)
 	headerConvert(h, HEADERCONV_COMPRESSFILELIST);
 }
 
-struct pkgdata_s {
-    const char *fn;
-    rpmRC rc;
-};
-
-static rpmRC handlePkgVS(struct rpmsinfo_s *sinfo, rpmRC rc, const char *msg, void *cbdata)
+static void loghdrmsg(struct rpmsinfo_s *sinfo, struct pkgdata_s *pkgdata,
+			const char *msg)
 {
-    struct pkgdata_s *pkgdata = cbdata;
     int lvl = RPMLOG_DEBUG;
-    char *vsmsg = rpmsinfoMsg(sinfo, rc, msg);
-    switch (rc) {
+    switch (sinfo->rc) {
     case RPMRC_OK:		/* Signature is OK. */
 	break;
     case RPMRC_NOTTRUSTED:	/* Signature is OK, but key is not trusted. */
@@ -286,26 +324,23 @@ static rpmRC handlePkgVS(struct rpmsinfo_s *sinfo, rpmRC rc, const char *msg, vo
 	break;
     }
 
-    rpmlog(lvl, "%s: %s\n", pkgdata->fn, vsmsg);
-
-    /* Remember actual return code, but don't override a previous failure */
-    if (rc && pkgdata->rc != RPMRC_FAIL)
-	pkgdata->rc = rc;
-
-    /* Preserve traditional behavior for now: only failure prevents read */
-    if (rc != RPMRC_FAIL)
-	rc = RPMRC_OK;
-
-    free(vsmsg);
-    return rc;
+    rpmlog(lvl, "%s: %s\n", pkgdata->fn, msg);
 }
 
 rpmRC rpmReadPackageFile(rpmts ts, FD_t fd, const char * fn, Header * hdrp)
 {
+    char *msg = NULL;
+    Header h = NULL;
+    Header sigh = NULL;
+    hdrblob blob = NULL;
+    hdrblob sigblob = NULL;
     rpmVSFlags vsflags = rpmtsVSFlags(ts) | RPMVSF_NEEDPAYLOAD;
     rpmKeyring keyring = rpmtsGetKeyring(ts, 1);
+    struct rpmvs_s *vs = rpmvsCreate(0, vsflags, keyring);
     struct pkgdata_s pkgdata = {
+	.msgfunc = loghdrmsg,
 	.fn = fn ? fn : Fdescr(fd),
+	.msg = NULL,
 	.rc = RPMRC_OK,
     };
 
@@ -313,13 +348,44 @@ rpmRC rpmReadPackageFile(rpmts ts, FD_t fd, const char * fn, Header * hdrp)
     if (hdrp)
 	*hdrp = NULL;
 
-    rpmRC rc = rpmpkgRead(keyring, vsflags, fd, handlePkgVS, &pkgdata, hdrp);
+    rpmRC rc = rpmpkgRead(vs, fd, &sigblob, &blob, &msg);
+    if (rc)
+	goto exit;
+
+    /* Actually all verify discovered signatures and digests */
+    rc = RPMRC_FAIL;
+    if (!rpmvsVerify(vs, RPMSIG_VERIFIABLE_TYPE, handleHdrVS, &pkgdata)) {
+	/* Finally import the headers and do whatever required retrofits etc */
+	if (hdrp) {
+	    if (hdrblobImport(sigblob, 0, &sigh, &msg))
+		goto exit;
+	    if (hdrblobImport(blob, 0, &h, &msg))
+		goto exit;
+
+	    /* Append (and remap) signature tags to the metadata. */
+	    headerMergeLegacySigs(h, sigh);
+	    applyRetrofits(h);
+
+	    /* Bump reference count for return. */
+	    *hdrp = headerLink(h);
+	}
+	rc = RPMRC_OK;
+    }
 
     /* If there was a "substatus" (NOKEY in practise), return that instead */
     if (rc == RPMRC_OK && pkgdata.rc)
 	rc = pkgdata.rc;
 
+exit:
+    if (rc && msg)
+	rpmlog(RPMLOG_ERR, "%s: %s\n", Fdescr(fd), msg);
+    hdrblobFree(sigblob);
+    hdrblobFree(blob);
+    headerFree(sigh);
+    headerFree(h);
     rpmKeyringFree(keyring);
+    rpmvsFree(vs);
+    free(msg);
 
     return rc;
 }
