@@ -44,6 +44,10 @@ const char * rpmcliRootDir = "/";
 
 rpmQueryFlags rpmcliQueryFlags;
 
+rpmVSFlags rpmcliVSFlags;
+
+int rpmcliVfyLevelMask;
+
 extern int _rpmio_debug;
 
 static int rpmcliInitialized = -1;
@@ -133,6 +137,10 @@ static void rpmcliAllArgCallback( poptContext con,
 	break;
     case POPT_DBPATH:
 	rpmcliConfigured();
+	if (arg && arg[0] != '/') {
+	    fprintf(stderr, _("arguments to --dbpath must begin with '/'\n"));
+	    exit(EXIT_FAILURE);
+	}
 	rpmPushMacro(NULL, "_dbpath", NULL, arg, RMIL_CMDLINE);
 	break;
     case POPT_SHOWVERSION:
@@ -160,14 +168,19 @@ static void rpmcliAllArgCallback( poptContext con,
 	
     case RPMCLI_POPT_NODIGEST:
 	rpmcliQueryFlags |= VERIFY_DIGEST;
+	rpmcliVSFlags |= RPMVSF_MASK_NODIGESTS;
+	rpmcliVfyLevelMask |= RPMSIG_DIGEST_TYPE;
 	break;
 
     case RPMCLI_POPT_NOSIGNATURE:
 	rpmcliQueryFlags |= VERIFY_SIGNATURE;
+	rpmcliVSFlags |= RPMVSF_MASK_NOSIGNATURES;
+	rpmcliVfyLevelMask |= RPMSIG_SIGNATURE_TYPE;
 	break;
 
     case RPMCLI_POPT_NOHDRCHK:
 	rpmcliQueryFlags |= VERIFY_HDRCHK;
+	rpmcliVSFlags |= RPMVSF_NOHDRCHK;
 	break;
 
     case RPMCLI_POPT_TARGETPLATFORM:
