@@ -140,6 +140,8 @@ static int rpmlibDeps(Header h)
     while (rpmdsNext(req) >= 0) {
 	if (!(rpmdsFlags(req) & RPMSENSE_RPMLIB))
 	    continue;
+	if (rpmdsFlags(req) & RPMSENSE_MISSINGOK)
+	    continue;
 	if (rpmdsSearch(rpmlib, req) < 0) {
 	    if (!nvr) {
 		nvr = headerGetAsString(h, RPMTAG_NEVRA);
@@ -490,6 +492,8 @@ void rpmpsmNotify(rpmpsm psm, int what, rpm_loff_t amount)
 {
     if (psm) {
 	int changed = 0;
+	if (amount > psm->total)
+	    amount = psm->total;
 	if (amount > psm->amount) {
 	    psm->amount = amount;
 	    changed = 1;
