@@ -136,44 +136,34 @@ static int readFile(FD_t fd, char **msg)
 struct vfydata_s {
     int seen;
     int bad;
+    int verbose;
 };
 
-static rpmRC formatVerbose(struct rpmsinfo_s *sinfo, rpmRC sigres, const char *result, void *cbdata)
-{
-    char *vsmsg = rpmsinfoMsg(sinfo, sigres, result);
-    rpmlog(RPMLOG_NOTICE, "    %s\n", vsmsg);
-    free(vsmsg);
-    return sigres;
-}
-
-/* Failures are uppercase, in parenthesis if NOKEY. Otherwise lowercase. */
-static rpmRC formatDefault(struct rpmsinfo_s *sinfo, rpmRC sigres, const char *result, void *cbdata)
+static int vfyCb(struct rpmsinfo_s *sinfo, void *cbdata)
 {
     struct vfydata_s *vd = cbdata;
     vd->seen |= sinfo->type;
-    if (sigres != RPMRC_OK)
+    if (sinfo->rc != RPMRC_OK)
 	vd->bad |= sinfo->type;
-    return sigres;
+    if (vd->verbose) {
+	char *vsmsg = rpmsinfoMsg(sinfo);
+	rpmlog(RPMLOG_NOTICE, "    %s\n", vsmsg);
+	free(vsmsg);
+    }
+    return 1;
 }
 
-rpmRC rpmpkgRead(rpmKeyring keyring, rpmVSFlags flags, FD_t fd,
-			    rpmsinfoCb cb, void *cbdata, Header *hdrp)
+rpmRC rpmpkgRead(struct rpmvs_s *vs, FD_t fd,
+		hdrblob *sigblobp, hdrblob *blobp, char **emsg)
 {
 
     char * msg = NULL;
     rpmRC xx, rc = RPMRC_FAIL; /* assume failure */
-    int failed = 0;
-    int leadtype = -1;
-    struct hdrblob_s sigblob, blob;
-    struct rpmvs_s *sigset = NULL;
-    Header h = NULL;
-    Header sigh = NULL;
+    hdrblob sigblob = hdrblobCreate();
+    hdrblob blob = hdrblobCreate();
     rpmDigestBundle bundle = fdGetBundle(fd, 1); /* freed with fd */
 
-    memset(&blob, 0, sizeof(blob));
-    memset(&sigblob, 0, sizeof(sigblob));
-
-    if ((xx = rpmLeadRead(fd, &leadtype, &msg)) != RPMRC_OK) {
+    if ((xx = rpmLeadRead(fd, &msg)) != RPMRC_OK) {
 	/* Avoid message spew on manifests */
 	if (xx == RPMRC_NOTFOUND)
 	    msg = _free(msg);
@@ -182,81 +172,76 @@ rpmRC rpmpkgRead(rpmKeyring keyring, rpmVSFlags flags, FD_t fd,
     }
 
     /* Read the signature header. Might not be in a contiguous region. */
-    if (hdrblobRead(fd, 1, 0, RPMTAG_HEADERSIGNATURES, &sigblob, &msg))
+    if (hdrblobRead(fd, 1, 0, RPMTAG_HEADERSIGNATURES, sigblob, &msg))
 	goto exit;
 
-    sigset = rpmvsCreate(&sigblob, flags);
+    rpmvsInit(vs, sigblob, bundle);
 
     /* Initialize digests ranging over the header */
-    rpmvsInitDigests(sigset, RPMSIG_HEADER, bundle);
+    rpmvsInitRange(vs, RPMSIG_HEADER);
 
     /* Read the header from the package. */
-    if (hdrblobRead(fd, 1, 1, RPMTAG_HEADERIMMUTABLE, &blob, &msg))
+    if (hdrblobRead(fd, 1, 1, RPMTAG_HEADERIMMUTABLE, blob, &msg))
 	goto exit;
 
-    /* Fish interesting tags from the main header. This is a bit hacky... */
-    if (!(flags & (RPMVSF_NOPAYLOAD|RPMVSF_NEEDPAYLOAD)))
-	rpmvsAppend(sigset, &blob, RPMTAG_PAYLOADDIGEST);
+    /* Finalize header range */
+    rpmvsFiniRange(vs, RPMSIG_HEADER);
 
-    /* Initialize digests ranging over the payload only */
-    rpmvsInitDigests(sigset, RPMSIG_PAYLOAD, bundle);
+    /* Unless disabled, read the payload, generating digest(s) on the fly. */
+    if (!(rpmvsFlags(vs) & RPMVSF_NEEDPAYLOAD)) {
+	/* Fish interesting tags from the main header. This is a bit hacky... */
+	rpmvsAppendTag(vs, blob, RPMTAG_PAYLOADDIGEST);
 
-    /* Verify header signatures and digests */
-    failed += rpmvsVerifyItems(sigset, (RPMSIG_HEADER), bundle, keyring, cb, cbdata);
+	/* Initialize digests ranging over the payload only */
+	rpmvsInitRange(vs, RPMSIG_PAYLOAD);
 
-    /* Unless disabled, read the file, generating digest(s) on the fly. */
-    if (!(flags & RPMVSF_NEEDPAYLOAD)) {
 	if (readFile(fd, &msg))
 	    goto exit;
+
+	/* Finalize payload range */
+	rpmvsFiniRange(vs, RPMSIG_PAYLOAD);
+	rpmvsFiniRange(vs, RPMSIG_HEADER|RPMSIG_PAYLOAD);
     }
 
-    /* Verify signatures and digests ranging over the payload */
-    failed += rpmvsVerifyItems(sigset, (RPMSIG_PAYLOAD), bundle,
-			keyring, cb, cbdata);
-    failed += rpmvsVerifyItems(sigset, (RPMSIG_HEADER|RPMSIG_PAYLOAD), bundle,
-			keyring, cb, cbdata);
-
-    if (failed == 0) {
-	/* Finally import the headers and do whatever required retrofits etc */
-	if (hdrp) {
-	    if (hdrblobImport(&sigblob, 0, &sigh, &msg))
-		goto exit;
-	    if (hdrblobImport(&blob, 0, &h, &msg))
-		goto exit;
-
-	    /* Append (and remap) signature tags to the metadata. */
-	    headerMergeLegacySigs(h, sigh);
-	    applyRetrofits(h, leadtype);
-
-	    /* Bump reference count for return. */
-	    *hdrp = headerLink(h);
-	}
-	rc = RPMRC_OK;
+    if (sigblobp && blobp) {
+	*sigblobp = sigblob;
+	*blobp = blob;
+	sigblob = NULL;
+	blob = NULL;
     }
+    rc = RPMRC_OK;
 
 exit:
-    if (rc && msg != NULL)
-	rpmlog(RPMLOG_ERR, "%s: %s\n", Fdescr(fd), msg);
-    free(msg);
-    free(sigblob.ei);
-    free(blob.ei);
-    headerFree(h);
-    headerFree(sigh);
-    rpmvsFree(sigset);
+    if (emsg)
+	*emsg = msg;
+    else
+	free(msg);
+    hdrblobFree(sigblob);
+    hdrblobFree(blob);
     return rc;
 }
 
-static int rpmpkgVerifySigs(rpmKeyring keyring, rpmVSFlags flags,
+static int rpmpkgVerifySigs(rpmKeyring keyring, int vfylevel, rpmVSFlags flags,
 			   FD_t fd, const char *fn)
 {
+    char *msg = NULL;
+    struct vfydata_s vd = { .seen = 0,
+			    .bad = 0,
+			    .verbose = rpmIsVerbose(),
+    };
     int rc;
-    if (rpmIsVerbose()) {
-	rpmlog(RPMLOG_NOTICE, "%s:\n", fn);
-	rc = rpmpkgRead(keyring, flags, fd, formatVerbose, NULL, NULL);
-    } else {
-	struct vfydata_s vd = { 0, 0 };
-	rpmlog(RPMLOG_NOTICE, "%s:", fn);
-	rc = rpmpkgRead(keyring, flags, fd, formatDefault, &vd, NULL);
+    struct rpmvs_s *vs = rpmvsCreate(vfylevel, flags, keyring);
+
+    rpmlog(RPMLOG_NOTICE, "%s:%s", fn, vd.verbose ? "\n" : "");
+
+    rc = rpmpkgRead(vs, fd, NULL, NULL, &msg);
+
+    if (rc)
+	goto exit;
+
+    rc = rpmvsVerify(vs, RPMSIG_VERIFIABLE_TYPE, vfyCb, &vd);
+
+    if (!vd.verbose) {
 	if (vd.seen & RPMSIG_DIGEST_TYPE) {
 	    rpmlog(RPMLOG_NOTICE, " %s", (vd.bad & RPMSIG_DIGEST_TYPE) ?
 					_("DIGESTS") : _("digests"));
@@ -267,6 +252,12 @@ static int rpmpkgVerifySigs(rpmKeyring keyring, rpmVSFlags flags,
 	}
 	rpmlog(RPMLOG_NOTICE, " %s\n", rc ? _("NOT OK") : _("OK"));
     }
+
+exit:
+    if (rc && msg)
+	rpmlog(RPMLOG_ERR, "%s: %s\n", Fdescr(fd), msg);
+    rpmvsFree(vs);
+    free(msg);
     return rc;
 }
 
@@ -276,7 +267,9 @@ int rpmVerifySignatures(QVA_t qva, rpmts ts, FD_t fd, const char * fn)
     int rc = 1; /* assume failure */
     if (ts && qva && fd && fn) {
 	rpmKeyring keyring = rpmtsGetKeyring(ts, 1);
-	rc = rpmpkgVerifySigs(keyring, qva->qva_flags, fd, fn);
+	rpmVSFlags vsflags = rpmtsVfyFlags(ts);
+	int vfylevel = rpmtsVfyLevel(ts);
+	rc = rpmpkgVerifySigs(keyring, vfylevel, vsflags, fd, fn);
     	rpmKeyringFree(keyring);
     }
     return rc;
@@ -287,12 +280,18 @@ int rpmcliVerifySignatures(rpmts ts, ARGV_const_t argv)
     const char * arg;
     int res = 0;
     rpmKeyring keyring = rpmtsGetKeyring(ts, 1);
-    rpmVSFlags vsflags = 0;
+    rpmVSFlags vsflags = rpmtsVfyFlags(ts);
+    int vfylevel = rpmtsVfyLevel(ts);
 
     if (rpmcliQueryFlags & QUERY_DIGEST)
 	vsflags |= _RPMVSF_NODIGESTS;
     if (rpmcliQueryFlags & QUERY_SIGNATURE)
 	vsflags |= _RPMVSF_NOSIGNATURES;
+    vsflags |= rpmcliVSFlags;
+    if (rpmcliVfyLevelMask) {
+	vfylevel &= ~rpmcliVfyLevelMask;
+	rpmtsSetVfyLevel(ts, vfylevel);
+    }
 
     while ((arg = *argv++) != NULL) {
 	FD_t fd = Fopen(arg, "r.ufdio");
@@ -300,7 +299,7 @@ int rpmcliVerifySignatures(rpmts ts, ARGV_const_t argv)
 	    rpmlog(RPMLOG_ERR, _("%s: open failed: %s\n"), 
 		     arg, Fstrerror(fd));
 	    res++;
-	} else if (rpmpkgVerifySigs(keyring, vsflags, fd, arg)) {
+	} else if (rpmpkgVerifySigs(keyring, vfylevel, vsflags, fd, arg)) {
 	    res++;
 	}
 
