@@ -100,6 +100,8 @@ enum rpmQVSources_e {
     RPMQV_WHATSUPPLEMENTS,	/*!< ... from supplements db search. */
     RPMQV_WHATENHANCES,		/*!< ... from enhances db search. */
     RPMQV_SPECBUILTRPMS,	/*!< ... from pkgs which would be built from spec */
+    RPMQV_WHATOBSOLETES,	/*!< ... from obsoletes db search. */
+    RPMQV_WHATCONFLICTS,	/*!< ... from conflicts db search. */
 };
 
 typedef rpmFlags rpmQVSources;
@@ -133,7 +135,8 @@ enum rpmQueryFlags_e {
     QUERY_FOR_DOCS	= (1 << 25),	/*!< query:  from --docfiles */
     QUERY_FOR_CONFIG	= (1 << 26),	/*!< query:  from --configfiles */
     QUERY_FOR_DUMPFILES	= (1 << 27),	/*!< query:  from --dump */
-    QUERY_FOR_LICENSE	= (1 << 28)	/*!< query:  from --licensefiles */
+    QUERY_FOR_LICENSE	= (1 << 28),	/*!< query:  from --licensefiles */
+    QUERY_FOR_ARTIFACT	= (1 << 29),	/*!< query:  from --artifacts */
 };
 
 typedef rpmFlags rpmQueryFlags;
@@ -147,6 +150,10 @@ typedef rpmFlags rpmQueryFlags;
  */
 extern rpmQueryFlags rpmcliQueryFlags;
 
+extern rpmVSFlags rpmcliVSFlags;
+
+extern int rpmcliVfyLevelMask;
+
 /** \ingroup rpmcli
  */
 typedef struct rpmQVKArguments_s * QVA_t;
@@ -255,7 +262,7 @@ int showVerifyPackage(QVA_t qva, rpmts ts, Header h);
 
 /**
  * Check package and header signatures.
- * @param qva		parsed query/verify options
+ * @param qva		unused
  * @param ts		transaction set
  * @param fd		package file handle
  * @param fn		package file name
