@@ -502,10 +502,14 @@ static int openDatabase(const char * prefix,
     if (db == NULL)
 	return 1;
 
+    db->db_next = rpmdbRock;
+    rpmdbRock = db;
+
     /* Try to ensure db home exists, error out if we can't even create */
     rc = rpmioMkpath(rpmdbHome(db), 0755, getuid(), getgid());
     if (rc == 0) {
-	if (rpmdbRock == NULL) {
+	/* Enable signal queue on the first db open */
+	if (db->db_next == NULL) {
 	    rpmsqActivate(1);
 	}
 
@@ -516,8 +520,6 @@ static int openDatabase(const char * prefix,
     if (rc || justCheck || dbp == NULL)
 	rpmdbClose(db);
     else {
-	db->db_next = rpmdbRock;
-	rpmdbRock = db;
         *dbp = db;
     }
 
@@ -2478,6 +2480,7 @@ static int rpmdbSetPermissions(char * src, char * dest)
 	rc += xx;
 	_free(filepath);
     }
+    closedir(dfd);
 
  exit:
     return rc;
@@ -2562,7 +2565,7 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
 	    /* Deleted entries are eliminated in legacy headers by copy. */
 	    if (headerIsEntry(h, RPMTAG_HEADERIMAGE)) {
 		Header nh = headerReload(headerCopy(h), RPMTAG_HEADERIMAGE);
-		rc = rpmdbAdd(newdb, h);
+		rc = rpmdbAdd(newdb, nh);
 		headerFree(nh);
 	    } else {
 		rc = rpmdbAdd(newdb, h);
