@@ -89,6 +89,7 @@ static rpmVSFlags setvsFlags(struct rpmInstallArguments_s * ia)
 	vsflags |= _RPMVSF_NOSIGNATURES;
     if (rpmcliQueryFlags & VERIFY_HDRCHK)
 	vsflags |= RPMVSF_NOHDRCHK;
+    vsflags |= rpmcliVSFlags;
 
     return vsflags;
 }
@@ -172,6 +173,7 @@ void * rpmShowProgress(const void * arg,
     case RPMCALLBACK_TRANS_PROGRESS:
     case RPMCALLBACK_INST_PROGRESS:
     case RPMCALLBACK_UNINST_PROGRESS:
+    case RPMCALLBACK_VERIFY_PROGRESS:
 	if (flags & INSTALL_PERCENT)
 	    fprintf(stdout, "%%%% %f\n", (double) (total
 				? ((((float) amount) / total) * 100)
@@ -182,6 +184,7 @@ void * rpmShowProgress(const void * arg,
 	break;
 
     case RPMCALLBACK_TRANS_START:
+    case RPMCALLBACK_VERIFY_START:
 	rpmcliHashesCurrent = 0;
 	rpmcliProgressTotal = 1;
 	rpmcliProgressCurrent = 0;
@@ -189,14 +192,18 @@ void * rpmShowProgress(const void * arg,
 	rpmcliProgressState = what;
 	if (!(flags & INSTALL_LABEL))
 	    break;
-	if (flags & INSTALL_HASH)
-	    fprintf(stdout, "%-38s", _("Preparing..."));
-	else
-	    fprintf(stdout, "%s\n", _("Preparing packages..."));
+	if (flags & INSTALL_HASH) {
+	    fprintf(stdout, "%-38s", (what == RPMCALLBACK_TRANS_START) ?
+		    _("Preparing...") : _("Verifying..."));
+	} else {
+	    fprintf(stdout, "%s\n", (what == RPMCALLBACK_TRANS_START) ?
+		    _("Preparing packages...") : _("Verifying packages..."));
+	}
 	(void) fflush(stdout);
 	break;
 
     case RPMCALLBACK_TRANS_STOP:
+    case RPMCALLBACK_VERIFY_STOP:
 	if (flags & INSTALL_HASH)
 	    printHash(1, 1);	/* Fixes "preparing..." progress bar */
 	rpmcliProgressTotal = rpmcliPackagesTotal;
@@ -411,14 +418,23 @@ int rpmInstall(rpmts ts, struct rpmInstallArguments_s * ia, ARGV_t fileArgv)
     rpmRelocation * relocations;
     char * fileURL = NULL;
     rpmVSFlags vsflags, ovsflags;
+    rpmVSFlags ovfyflags;
     int rc;
     int i;
 
     vsflags = setvsFlags(ia);
     ovsflags = rpmtsSetVSFlags(ts, (vsflags | RPMVSF_NEEDPAYLOAD));
+    /* for rpm cli, --nosignature/--nodigest applies to both vs and vfyflags */
+    ovfyflags = rpmtsSetVfyFlags(ts, (rpmtsVfyFlags(ts) | rpmcliVSFlags));
 
     if (fileArgv == NULL) goto exit;
 
+    if (rpmcliVfyLevelMask) {
+	int vfylevel = rpmtsVfyLevel(ts);
+	vfylevel &= ~rpmcliVfyLevelMask;
+	rpmtsSetVfyLevel(ts, vfylevel);
+    }
+
     (void) rpmtsSetFlags(ts, ia->transFlags);
 
     relocations = ia->relocations;
@@ -659,6 +675,7 @@ exit:
 
     rpmtsEmpty(ts);
     rpmtsSetVSFlags(ts, ovsflags);
+    rpmtsSetVfyFlags(ts, ovfyflags);
 
     return rc;
 }
@@ -721,7 +738,7 @@ exit:
     rpmtsEmpty(ts);
     rpmtsSetVSFlags(ts, ovsflags);
 
-    return numFailed;
+    return (numFailed < 0) ? numPackages : numFailed;
 }
 
 int rpmInstallSource(rpmts ts, const char * arg,
