@@ -11,6 +11,7 @@
 #include "lib/rpmdb_internal.h"
 #include "lib/rpmds_internal.h"
 #include "lib/rpmfi_internal.h"
+#include "lib/rpmte_internal.h"
 #include "lib/rpmchroot.h"
 
 #define TRIGGER_PRIORITY_BOUND 10000
@@ -106,7 +107,6 @@ void rpmtriggersPrepPostUnTransFileTrigs(rpmts ts, rpmte te)
     rpmfiles files;
     rpmds rpmdsTriggers;
     rpmds rpmdsTrigger;
-    int tix = 0;
 
     ii = rpmdbIndexIteratorInit(rpmtsGetRdb(ts), RPMDBI_TRANSFILETRIGGERNAME);
     mi = rpmdbNewIterator(rpmtsGetRdb(ts), RPMDBI_PACKAGES);
@@ -117,12 +117,15 @@ void rpmtriggersPrepPostUnTransFileTrigs(rpmts ts, rpmte te)
 	char pfx[keylen + 1];
 	memcpy(pfx, key, keylen);
 	pfx[keylen] = '\0';
-	/* Check if file trigger matches any file in this te */
+	/* Check if file trigger matches any installed file in this te */
 	rpmfi fi = rpmfilesFindPrefix(files, pfx);
-	if (rpmfiFC(fi) > 0) {
-	    /* If yes then store it */
-	    rpmdbAppendIterator(mi, rpmdbIndexIteratorPkgOffsets(ii),
+	while (rpmfiNext(fi) >= 0) {
+	    if (RPMFILE_IS_INSTALLED(rpmfiFState(fi))) {
+		/* If yes then store it */
+		rpmdbAppendIterator(mi, rpmdbIndexIteratorPkgOffsets(ii),
 				rpmdbIndexIteratorNumPkgs(ii));
+		break;
+	    }
 	}
 	rpmfiFree(fi);
     }
@@ -131,6 +134,7 @@ void rpmtriggersPrepPostUnTransFileTrigs(rpmts ts, rpmte te)
     if (rpmdbGetIteratorCount(mi)) {
 	/* Filter triggers and save only trans postun triggers into ts */
 	while ((trigH = rpmdbNextIterator(mi)) != NULL) {
+	    int tix = 0;
 	    rpmdsTriggers = rpmdsNew(trigH, RPMTAG_TRANSFILETRIGGERNAME, 0);
 	    while ((rpmdsTrigger = rpmdsFilterTi(rpmdsTriggers, tix))) {
 		if ((rpmdsNext(rpmdsTrigger) >= 0) &&
@@ -150,6 +154,7 @@ void rpmtriggersPrepPostUnTransFileTrigs(rpmts ts, rpmte te)
 	}
     }
     rpmdbFreeIterator(mi);
+    rpmfilesFree(files);
 }
 
 int runPostUnTransFileTrigs(rpmts ts)
@@ -230,7 +235,7 @@ static rpmfiles rpmtsNextFiles(rpmts ts, rpmdbMatchIterator mi)
 	h = rpmdbNextIterator(mi);
 	if (h) {
 	    files = rpmfilesNew(pool, h, RPMTAG_BASENAMES,
-				RPMFI_FLAGS_ONLY_FILENAMES);
+				RPMFI_FLAGS_FILETRIGGER);
 	}
     }
 
@@ -243,17 +248,19 @@ typedef struct matchFilesIter_s {
     rpmds rpmdsTrigger;
     rpmfiles files;
     rpmfi fi;
+    rpmfs fs;
     const char *pfx;
     rpmdbMatchIterator pi;
     packageHash tranPkgs;
 } *matchFilesIter;
 
-static matchFilesIter matchFilesIterator(rpmds trigger, rpmfiles files)
+static matchFilesIter matchFilesIterator(rpmds trigger, rpmfiles files, rpmte te)
 {
     matchFilesIter mfi = xcalloc(1, sizeof(*mfi));
     rpmdsInit(trigger);
     mfi->rpmdsTrigger = trigger;
     mfi->files = rpmfilesLink(files);
+    mfi->fs = rpmteGetFileStates(te);
     return mfi;
 }
 
@@ -289,11 +296,13 @@ static const char *matchFilesNext(matchFilesIter mfi)
     if (!mfi->ts)
 	do {
 	    /* Get next file from mfi->fi */
-	    rpmfiNext(mfi->fi);
-	    matchFile = rpmfiFN(mfi->fi);
-	    if (strlen(matchFile))
-		break;
 	    matchFile = NULL;
+	    while (matchFile == NULL && rpmfiNext(mfi->fi) >= 0) {
+		if (!XFA_SKIPPING(rpmfsGetAction(mfi->fs, rpmfiFX(mfi->fi))))
+		    matchFile = rpmfiFN(mfi->fi);
+	    }
+	    if (matchFile)
+		break;
 
 	    /* If we are done with current mfi->fi, create mfi->fi for next prefix */
 	    fx = rpmdsNext(mfi->rpmdsTrigger);
@@ -305,11 +314,13 @@ static const char *matchFilesNext(matchFilesIter mfi)
     /* or we iterate over files in rpmdb */
     else
 	do {
-	    rpmfiNext(mfi->fi);
-	    matchFile = rpmfiFN(mfi->fi);
-	    if (strlen(matchFile))
-		break;
 	    matchFile = NULL;
+	    while (matchFile == NULL && rpmfiNext(mfi->fi) >= 0) {
+		if (RPMFILE_IS_INSTALLED(rpmfiFState(mfi->fi)))
+		    matchFile = rpmfiFN(mfi->fi);
+	    }
+	    if (matchFile)
+		break;
 
 	    /* If we are done with current mfi->fi, create mfi->fi for next package */
 	    rpmfilesFree(mfi->files);
@@ -327,6 +338,8 @@ static const char *matchFilesNext(matchFilesIter mfi)
 						RPMDBI_DIRNAMES, mfi->pfx, 0);
 
 	    rpmdbFilterIterator(mfi->pi, mfi->tranPkgs, 0);
+	    /* Only walk through each header with matches once */
+	    rpmdbUniqIterator(mfi->pi);
 
 	} while (fx >= 0);
 
@@ -393,7 +406,7 @@ static int runHandleTriggersInPkg(rpmts ts, rpmte te, Header h,
 	    case 0:
 		/* Create iterator over files in te that this trigger matches */
 		files = rpmteFiles(te);
-		mfi = matchFilesIterator(rpmdsTrigger, files);
+		mfi = matchFilesIterator(rpmdsTrigger, files, te);
 		break;
 	    case 1:
 		/* Create iterator over files in ts that this trigger matches */
@@ -452,7 +465,7 @@ static int matchFilesInPkg(rpmts ts, rpmte te, const char *pfx,
     return rc;
 }
 
-/* Return true if any added/removed file in ts starts with pfx */
+/* Return number of added/removed files starting with pfx in transaction */
 static int matchFilesInTran(rpmts ts, rpmte te, const char *pfx,
 			    rpmsenseFlags sense)
 {
