@@ -191,28 +191,65 @@ exit:
     return rc;
 }
 
+typedef enum tMode_e {
+    NORMALTRIGGER     = 0,
+    FILETRIGGER       = 1,
+    TRANSFILETRIGGER  = 2,
+} tMode;
+
 /**
  * Retrieve trigger info.
+ * @param mode		type of trigger (see tMode_e)
  * @param h		header
  * @retval td		tag data container
+ * @param hgflags	header get flags
  * @return		1 on success
  */
-static int triggercondsTag(Header h, rpmtd td, headerGetFlags hgflags)
+static int triggercondsTagFor(tMode mode, Header h, rpmtd td,
+				headerGetFlags hgflags)
 {
     uint32_t * indices;
     int i, j;
     char ** conds;
     struct rpmtd_s nametd, indextd, flagtd, versiontd, scripttd;
     int hgeflags = HEADERGET_MINMEM;
+    rpmTagVal triggername, triggerindex, triggerflags;
+    rpmTagVal triggerversion, triggerscripts;
 
-    if (!headerGet(h, RPMTAG_TRIGGERNAME, &nametd, hgeflags)) {
+    switch (mode) {
+	case NORMALTRIGGER:
+	    triggername = RPMTAG_TRIGGERNAME;
+	    triggerindex = RPMTAG_TRIGGERINDEX;
+	    triggerflags = RPMTAG_TRIGGERFLAGS;
+	    triggerversion = RPMTAG_TRIGGERVERSION;
+	    triggerscripts = RPMTAG_TRIGGERSCRIPTS;
+	    break;
+	case FILETRIGGER:
+	    triggername = RPMTAG_FILETRIGGERNAME;
+	    triggerindex = RPMTAG_FILETRIGGERINDEX;
+	    triggerflags = RPMTAG_FILETRIGGERFLAGS;
+	    triggerversion = RPMTAG_FILETRIGGERVERSION;
+	    triggerscripts = RPMTAG_FILETRIGGERSCRIPTS;
+	    break;
+	case TRANSFILETRIGGER:
+	    triggername = RPMTAG_TRANSFILETRIGGERNAME;
+	    triggerindex = RPMTAG_TRANSFILETRIGGERINDEX;
+	    triggerflags = RPMTAG_TRANSFILETRIGGERFLAGS;
+	    triggerversion = RPMTAG_TRANSFILETRIGGERVERSION;
+	    triggerscripts = RPMTAG_TRANSFILETRIGGERSCRIPTS;
+	    break;
+	default:
+	    return 0;
+    }
+
+    if (!headerGet(h, triggername, &nametd, hgeflags)) {
 	return 0;
     }
 
-    headerGet(h, RPMTAG_TRIGGERINDEX, &indextd, hgeflags);
-    headerGet(h, RPMTAG_TRIGGERFLAGS, &flagtd, hgeflags);
-    headerGet(h, RPMTAG_TRIGGERVERSION, &versiontd, hgeflags);
-    headerGet(h, RPMTAG_TRIGGERSCRIPTS, &scripttd, hgeflags);
+    headerGet(h, triggerindex, &indextd, hgeflags);
+    headerGet(h, triggerflags, &flagtd, hgeflags);
+    headerGet(h, triggerversion, &versiontd, hgeflags);
+    headerGet(h, triggerscripts, &scripttd, hgeflags);
 
     td->type = RPM_STRING_ARRAY_TYPE;
     td->flags = RPMTD_ALLOCED | RPMTD_PTR_ALLOCED;
@@ -261,24 +298,63 @@ static int triggercondsTag(Header h, rpmtd td, headerGetFlags hgflags)
     return 1;
 }
 
+static int triggercondsTag(Header h, rpmtd td, headerGetFlags hgflags)
+{
+    return triggercondsTagFor(NORMALTRIGGER, h, td, hgflags);
+}
+
+static int filetriggercondsTag(Header h, rpmtd td, headerGetFlags hgflags)
+{
+    return triggercondsTagFor(FILETRIGGER, h, td, hgflags);
+}
+
+static int transfiletriggercondsTag(Header h, rpmtd td, headerGetFlags hgflags)
+{
+    return triggercondsTagFor(TRANSFILETRIGGER, h, td, hgflags);
+}
+
 /**
  * Retrieve trigger type info.
+ * @param mode		type of trigger (see tMode_e)
  * @param h		header
  * @retval td		tag data container
+ * @param hgflags	header get flags
  * @return		1 on success
  */
-static int triggertypeTag(Header h, rpmtd td, headerGetFlags hgflags)
+static int triggertypeTagFor(tMode mode, Header h, rpmtd td,
+				headerGetFlags hgflags)
 {
     int i;
     char ** conds;
     struct rpmtd_s indices, flags, scripts;
+    rpmTagVal triggerindex, triggerflags, triggerscripts;
 
-    if (!headerGet(h, RPMTAG_TRIGGERINDEX, &indices, HEADERGET_MINMEM)) {
+    switch (mode) {
+	case NORMALTRIGGER:
+	    triggerindex = RPMTAG_TRIGGERINDEX;
+	    triggerflags = RPMTAG_TRIGGERFLAGS;
+	    triggerscripts = RPMTAG_TRIGGERSCRIPTS;
+	    break;
+	case FILETRIGGER:
+	    triggerindex = RPMTAG_FILETRIGGERINDEX;
+	    triggerflags = RPMTAG_FILETRIGGERFLAGS;
+	    triggerscripts = RPMTAG_FILETRIGGERSCRIPTS;
+	    break;
+	case TRANSFILETRIGGER:
+	    triggerindex = RPMTAG_TRANSFILETRIGGERINDEX;
+	    triggerflags = RPMTAG_TRANSFILETRIGGERFLAGS;
+	    triggerscripts = RPMTAG_TRANSFILETRIGGERSCRIPTS;
+	    break;
+	default:
+	   return 0;
+    }
+
+    if (!headerGet(h, triggerindex, &indices, HEADERGET_MINMEM)) {
 	return 0;
     }
 
-    headerGet(h, RPMTAG_TRIGGERFLAGS, &flags, HEADERGET_MINMEM);
-    headerGet(h, RPMTAG_TRIGGERSCRIPTS, &scripts, HEADERGET_MINMEM);
+    headerGet(h, triggerflags, &flags, HEADERGET_MINMEM);
+    headerGet(h, triggerscripts, &scripts, HEADERGET_MINMEM);
 
     td->flags = RPMTD_ALLOCED | RPMTD_PTR_ALLOCED;
     td->count = rpmtdCount(&scripts);
@@ -314,20 +390,38 @@ static int triggertypeTag(Header h, rpmtd td, headerGetFlags hgflags)
     return 1;
 }
 
+static int triggertypeTag(Header h, rpmtd td, headerGetFlags hgflags)
+{
+    return triggertypeTagFor(NORMALTRIGGER, h, td, hgflags);
+}
+
+static int filetriggertypeTag(Header h, rpmtd td, headerGetFlags hgflags)
+{
+    return triggertypeTagFor(FILETRIGGER, h, td, hgflags);
+}
+
+static int transfiletriggertypeTag(Header h, rpmtd td, headerGetFlags hgflags)
+{
+    return triggertypeTagFor(TRANSFILETRIGGER, h, td, hgflags);
+}
+
 /**
  * Retrieve installed file paths.
  * @param h		header
  * @retval td		tag data container
+ * @param hgflags	header get flags
  * @return		1 on success
  */
 static int instfilenamesTag(Header h, rpmtd td, headerGetFlags hgflags)
 {
     return fnTag(h, RPMTAG_BASENAMES, 1, td);
 }
+
 /**
  * Retrieve file paths.
  * @param h		header
  * @retval td		tag data container
+ * @param hgflags	header get flags
  * @return		1 on success
  */
 static int filenamesTag(Header h, rpmtd td, headerGetFlags hgflags)
@@ -339,6 +433,7 @@ static int filenamesTag(Header h, rpmtd td, headerGetFlags hgflags)
  * Retrieve original file paths (wrt relocation).
  * @param h		header
  * @retval td		tag data container
+ * @param hgflags	header get flags
  * @return		1 on success
  */
 static int origfilenamesTag(Header h, rpmtd td, headerGetFlags hgflags)
@@ -389,6 +484,7 @@ static char *makeFClass(rpmfi fi)
  * Retrieve/generate file classes.
  * @param h		header
  * @retval td		tag data container
+ * @param hgflags	header get flags
  * @return		1 on success
  */
 static int fileclassTag(Header h, rpmtd td, headerGetFlags hgflags)
@@ -419,6 +515,7 @@ static int fileclassTag(Header h, rpmtd td, headerGetFlags hgflags)
  * Retrieve file provides.
  * @param h		header
  * @retval td		tag data container
+ * @param hgflags	header get flags
  * @return		1 on success
  */
 static int fileprovideTag(Header h, rpmtd td, headerGetFlags hgflags)
@@ -430,6 +527,7 @@ static int fileprovideTag(Header h, rpmtd td, headerGetFlags hgflags)
  * Retrieve file requires.
  * @param h		header
  * @retval td		tag data container
+ * @param hgflags	header get flags
  * @return		1 on success
  */
 static int filerequireTag(Header h, rpmtd td, headerGetFlags hgflags)
@@ -451,6 +549,7 @@ static const char * const _macro_i18ndomains = "%{?_i18ndomains}";
  * @param h		header
  * @param tag		tag
  * @retval td		tag data container
+ * @param hgflags	header get flags
  * @return		1 on success
  */
 static int i18nTag(Header h, rpmTag tag, rpmtd td, headerGetFlags hgflags)
@@ -515,6 +614,7 @@ static int i18nTag(Header h, rpmTag tag, rpmtd td, headerGetFlags hgflags)
  * Retrieve summary text.
  * @param h		header
  * @retval td		tag data container
+ * @param hgflags	header get flags
  * @return		1 on success
  */
 static int summaryTag(Header h, rpmtd td, headerGetFlags hgflags)
@@ -526,6 +626,7 @@ static int summaryTag(Header h, rpmtd td, headerGetFlags hgflags)
  * Retrieve description text.
  * @param h		header
  * @retval td		tag data container
+ * @param hgflags	header get flags
  * @return		1 on success
  */
 static int descriptionTag(Header h, rpmtd td, headerGetFlags hgflags)
@@ -537,6 +638,7 @@ static int descriptionTag(Header h, rpmtd td, headerGetFlags hgflags)
  * Retrieve group text.
  * @param h		header
  * @retval td		tag data container
+ * @param hgflags	header get flags
  * @return		1 on success
  */
 static int groupTag(Header h, rpmtd td, headerGetFlags hgflags)
@@ -583,6 +685,7 @@ static int get64(Header h, rpmtd td, rpmTag newtag, rpmTag oldtag)
  * Retrieve file sizes as 64bit regardless of how they're stored.
  * @param h		header
  * @retval td		tag data container
+ * @param hgflags	header get flags
  * @return		1 on success
  */
 static int longfilesizesTag(Header h, rpmtd td, headerGetFlags hgflags)
@@ -859,7 +962,11 @@ static const struct headerTagFunc_s rpmHeaderTagExtensions[] = {
     { RPMTAG_FILEPROVIDE,	fileprovideTag },
     { RPMTAG_FILEREQUIRE,	filerequireTag },
     { RPMTAG_TRIGGERCONDS,	triggercondsTag },
+    { RPMTAG_FILETRIGGERCONDS,	filetriggercondsTag },
+    { RPMTAG_TRANSFILETRIGGERCONDS,	transfiletriggercondsTag },
     { RPMTAG_TRIGGERTYPE,	triggertypeTag },
+    { RPMTAG_FILETRIGGERTYPE,	filetriggertypeTag },
+    { RPMTAG_TRANSFILETRIGGERTYPE,	transfiletriggertypeTag },
     { RPMTAG_LONGFILESIZES,	longfilesizesTag },
     { RPMTAG_LONGARCHIVESIZE,	longarchivesizeTag },
     { RPMTAG_LONGSIZE,		longsizeTag },
