@@ -15,6 +15,7 @@
 #include <rpm/rpmfileutil.h>
 #include <rpm/rpmstring.h>
 #include <rpm/rpmsq.h>
+#include <rpm/rpmkeyring.h>
 
 #include "lib/fprint.h"
 #include "lib/misc.h"
@@ -24,6 +25,7 @@
 #include "lib/rpmfi_internal.h"	/* only internal apis */
 #include "lib/rpmte_internal.h"	/* only internal apis */
 #include "lib/rpmts_internal.h"
+#include "lib/rpmvs.h"
 #include "rpmio/rpmhook.h"
 #include "lib/rpmtriggers.h"
 
@@ -233,11 +235,11 @@ static void rpmtsUpdateDSI(const rpmts ts, dev_t dev, const char *dirName,
 	dsi->bneeded += bneeded;
 	dsi->ineeded++;
 	if (prevSize) {
-	    dsi->bdelta += BLOCK_ROUND(prevSize, dsi->bsize);
+	    dsi->bdelta += BLOCK_ROUND(prevSize - 1, dsi->bsize);
 	    dsi->idelta++;
 	}
 	if (fixupSize) {
-	    dsi->bdelta += BLOCK_ROUND(fixupSize, dsi->bsize);
+	    dsi->bdelta += BLOCK_ROUND(fixupSize - 1, dsi->bsize);
 	    dsi->idelta++;
 	}
 
@@ -414,6 +416,9 @@ static void handleInstInstalledFile(const rpmts ts, rpmte p, rpmfiles fi, int fx
 {
     rpmfs fs = rpmteGetFileStates(p);
     int isCfgFile = ((rpmfilesFFlags(otherFi, ofx) | rpmfilesFFlags(fi, fx)) & RPMFILE_CONFIG);
+    rpm_loff_t otherFileSize;
+    int nlink;
+    const int *links;
 
     if (XFA_SKIPPING(rpmfsGetAction(fs, fx)))
 	return;
@@ -483,7 +488,15 @@ static void handleInstInstalledFile(const rpmts ts, rpmte p, rpmfiles fi, int fx
 	}
     }
 
-    rpmfilesSetFReplacedSize(fi, fx, rpmfilesFSize(otherFi, ofx));
+    otherFileSize = rpmfilesFSize(otherFi, ofx);
+
+    /* Only account for the last file of a hardlink set */
+    nlink = rpmfilesFLinks(otherFi, ofx, &links);
+    if (nlink > 1 && links[nlink - 1] != ofx)
+	otherFileSize = 0;
+
+    /* Add one to make sure the size is not zero */
+    rpmfilesSetFReplacedSize(fi, fx, otherFileSize + 1);
 }
 
 /**
@@ -508,6 +521,9 @@ static void handleOverlappedFiles(rpmts ts, fingerPrintCache fpc, rpmte p, rpmfi
 	rpmfileAttrs FFlags;
 	struct rpmffi_s * recs;
 	int numRecs;
+	rpm_loff_t fileSize;
+	int nlink;
+	const int *links;
 
 	if (XFA_SKIPPING(rpmfsGetAction(fs, i)))
 	    continue;
@@ -630,7 +646,8 @@ assert(otherFi != NULL);
 		break;
 
 	    /* Try to get the disk accounting correct even if a conflict. */
-	    fixupSize = rpmfilesFSize(otherFi, otherFileNum);
+	    /* Add one to make sure the size is not zero */
+	    fixupSize = rpmfilesFSize(otherFi, otherFileNum) + 1;
 
 	    if (rpmfilesConfigConflict(fi, i)) {
 		/* Here is an overlapped  pre-existing config file. */
@@ -675,9 +692,16 @@ assert(otherFi != NULL);
 	}
 	rpmfilesFree(otherFi);
 
+	fileSize = rpmfilesFSize(fi, i);
+	nlink = rpmfilesFLinks(fi, i, &links);
+	if (nlink > 1 && links[nlink - 1] != i) {
+	    /* Only account for the last file of a hardlink set */
+	    fileSize = 0;
+	    fixupSize = fixupSize ? 1 : 0;
+	}
 	/* Update disk space info for a file. */
 	rpmtsUpdateDSI(ts, fpEntryDev(fpc, fiFps), fpEntryDir(fpc, fiFps),
-		       rpmfilesFSize(fi, i), rpmfilesFReplacedSize(fi, i),
+		       fileSize, rpmfilesFReplacedSize(fi, i),
 		       fixupSize, rpmfsGetAction(fs, i));
 
     }
@@ -1158,6 +1182,96 @@ void checkInstalledFiles(rpmts ts, uint64_t fileCount, fingerPrintCache fpc)
 #define badArch(_a) (rpmMachineScore(RPM_MACHTABLE_INSTARCH, (_a)) == 0)
 #define badOs(_a) (rpmMachineScore(RPM_MACHTABLE_INSTOS, (_a)) == 0)
 
+static rpm_loff_t countPkgs(rpmts ts, rpmElementTypes types)
+{
+    rpm_loff_t npkgs = 0;
+    rpmtsi pi = rpmtsiInit(ts);
+    rpmte p;
+    while ((p = rpmtsiNext(pi, types)))
+	npkgs++;
+    pi = rpmtsiFree(pi);
+    return npkgs;
+}
+
+struct vfydata_s {
+    char *msg;
+    int vfylevel;
+};
+
+static int vfyCb(struct rpmsinfo_s *sinfo, void *cbdata)
+{
+    struct vfydata_s *vd = cbdata;
+    switch (sinfo->rc) {
+    case RPMRC_OK:
+	break;
+    case RPMRC_NOTFOUND:
+	vd->msg = xstrdup((sinfo->type == RPMSIG_SIGNATURE_TYPE) ?
+			  _("no signature") : _("no digest"));
+	break;
+    case RPMRC_NOKEY:
+	/*
+	 * Legacy compat: if signatures are not required, install must
+	 * succeed despite missing key.
+	 */
+	if (!(vd->vfylevel & RPMSIG_SIGNATURE_TYPE))
+	    sinfo->rc = RPMRC_OK;
+	/* fallthrough */
+    default:
+	if (sinfo->rc)
+	    vd->msg = rpmsinfoMsg(sinfo);
+	break;
+    }
+    return (sinfo->rc == 0);
+}
+
+static int verifyPackageFiles(rpmts ts, rpm_loff_t total)
+{
+    int rc = 0;
+    rpmKeyring keyring = rpmtsGetKeyring(ts, 0);
+    rpmtsi pi = NULL;
+    rpmte p;
+    rpm_loff_t oc = 0;
+    rpmVSFlags vsflags = rpmtsVfyFlags(ts);
+    int vfylevel = rpmtsVfyLevel(ts);
+
+    rpmtsNotify(ts, NULL, RPMCALLBACK_VERIFY_START, 0, total);
+
+    (void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_VERIFY), 0);
+
+    pi = rpmtsiInit(ts);
+    while ((p = rpmtsiNext(pi, TR_ADDED))) {
+	struct rpmvs_s *vs = rpmvsCreate(vfylevel, vsflags, keyring);
+	struct vfydata_s vd = {
+	    .msg = NULL,
+	    .vfylevel = vfylevel,
+	};
+	rpmRC prc = RPMRC_FAIL;
+
+	rpmtsNotify(ts, p, RPMCALLBACK_VERIFY_PROGRESS, oc++, total);
+	FD_t fd = rpmtsNotify(ts, p, RPMCALLBACK_INST_OPEN_FILE, 0, 0);
+	if (fd != NULL) {
+	    prc = rpmpkgRead(vs, fd, NULL, NULL, &vd.msg);
+	    rpmtsNotify(ts, p, RPMCALLBACK_INST_CLOSE_FILE, 0, 0);
+	}
+
+	if (prc == RPMRC_OK)
+	    prc = rpmvsVerify(vs, RPMSIG_VERIFIABLE_TYPE, vfyCb, &vd);
+
+	if (prc)
+	    rpmteAddProblem(p, RPMPROB_VERIFY, NULL, vd.msg, 0);
+
+	vd.msg = _free(vd.msg);
+	rpmvsFree(vs);
+    }
+    rpmtsNotify(ts, NULL, RPMCALLBACK_VERIFY_STOP, total, total);
+
+    (void) rpmswExit(rpmtsOp(ts, RPMTS_OP_VERIFY), 0);
+
+    rpmtsiFree(pi);
+    rpmKeyringFree(keyring);
+    return rc;
+}
+
 /*
  * For packages being installed:
  * - verify package arch/os.
@@ -1168,12 +1282,17 @@ static rpmps checkProblems(rpmts ts)
     rpm_color_t tscolor = rpmtsColor(ts);
     rpmprobFilterFlags probFilter = rpmtsFilterFlags(ts);
     rpmstrPool tspool = rpmtsPool(ts);
-    rpmtsi pi = rpmtsiInit(ts);
+    rpm_loff_t npkgs = countPkgs(ts, TR_ADDED);
+    rpmtsi pi;
     rpmte p;
 
+    if (npkgs == 0)
+	goto exit;
+
     /* The ordering doesn't matter here */
     /* XXX Only added packages need be checked. */
-    rpmlog(RPMLOG_DEBUG, "sanity checking %d elements\n", rpmtsNElements(ts));
+    rpmlog(RPMLOG_DEBUG, "sanity checking %lu elements\n", npkgs);
+    pi = rpmtsiInit(ts);
     while ((p = rpmtsiNext(pi, TR_ADDED)) != NULL) {
 
 	if (!(probFilter & RPMPROB_FILTER_IGNOREARCH) && badArch(rpmteA(p)))
@@ -1214,6 +1333,11 @@ static rpmps checkProblems(rpmts ts)
 	    rpmteAddRelocProblems(p);
     }
     rpmtsiFree(pi);
+
+    if (rpmtsVfyLevel(ts) && !(probFilter & RPMPROB_FILTER_VERIFY))
+	verifyPackageFiles(ts, npkgs);
+
+exit:
     return rpmtsProblems(ts);
 }
 
@@ -1504,6 +1628,7 @@ int rpmtsRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
     rpmtxn txn = NULL;
     rpmps tsprobs = NULL;
     int TsmPreDone = 0; /* TsmPre hook hasn't been called */
+    int nelem = rpmtsNElements(ts);
     /* Ignore SIGPIPE for the duration of transaction */
     rpmsqAction_t oact = rpmsqSetAction(SIGPIPE, RPMSQ_IGN);
     
@@ -1511,7 +1636,7 @@ int rpmtsRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
     mode_t oldmask = umask(022);
 
     /* Empty transaction, nothing to do */
-    if (rpmtsNElements(ts) <= 0) {
+    if (nelem <= 0) {
 	rc = 0;
 	goto exit;
     }
@@ -1557,7 +1682,7 @@ int rpmtsRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
 
      /* If unfiltered problems exist, free memory and return. */
     if ((rpmtsFlags(ts) & RPMTRANS_FLAG_BUILD_PROBS) || (rpmpsNumProblems(tsprobs))) {
-	rc = tsmem->orderCount;
+	rc = nelem;
 	goto exit;
     }
 
