@@ -41,8 +41,8 @@
  * -	rpm.RPMVSF_NORSA	if set, don't check header+payload RSA signature
  *
  * For convenience, there are the following masks:
- * -    rpm._RPMVSF_NODIGESTS		if set, don't check digest(s).
- * -    rpm._RPMVSF_NOSIGNATURES	if set, don't check signature(s).
+ * -    rpm.RPMVSF_MASK_NODIGESTS		if set, don't check digest(s).
+ * -    rpm.RPMVSF_MASK_NOSIGNATURES	if set, don't check signature(s).
  *
  * A rpm.ts object has the following methods:
  *
@@ -495,6 +495,8 @@ rpmtsCallback(const void * hd, const rpmCallbackType what,
 
     if (cbInfo->cb == Py_None) return NULL;
 
+    PyEval_RestoreThread(cbInfo->_save);
+
     /* Synthesize a python object for callback (if necessary). */
     if (pkgObj == NULL) {
 	if (h) {
@@ -506,8 +508,6 @@ rpmtsCallback(const void * hd, const rpmCallbackType what,
     } else
 	Py_INCREF(pkgObj);
 
-    PyEval_RestoreThread(cbInfo->_save);
-
     args = Py_BuildValue("(iLLOO)", what, amount, total, pkgObj, cbInfo->data);
     result = PyEval_CallObject(cbInfo->cb, args);
     Py_DECREF(args);
@@ -916,6 +916,24 @@ static int rpmts_set_vsflags(rpmtsObject *s, PyObject *value, void *closure)
     return 0;
 }
 
+static int rpmts_set_vfyflags(rpmtsObject *s, PyObject *value, void *closure)
+{
+    rpmVSFlags flags;
+    if (!PyArg_Parse(value, "i", &flags)) return -1;
+
+    /* TODO: validate the bits */
+    rpmtsSetVfyFlags(s->ts, flags);
+    return 0;
+}
+
+static int rpmts_set_vfylevel(rpmtsObject *s, PyObject *value, void *closure)
+{
+    int vfylevel;
+    if (!PyArg_Parse(value, "i", &vfylevel)) return -1;
+    rpmtsSetVfyLevel(s->ts, vfylevel);
+    return 0;
+}
+
 static PyObject *rpmts_get_flags(rpmtsObject *s, void *closure)
 {
     return Py_BuildValue("i", rpmtsFlags(s->ts));
@@ -926,6 +944,16 @@ static PyObject *rpmts_get_vsflags(rpmtsObject *s, void *closure)
     return Py_BuildValue("i", rpmtsVSFlags(s->ts));
 }
 
+static PyObject *rpmts_get_vfyflags(rpmtsObject *s, void *closure)
+{
+    return Py_BuildValue("i", rpmtsVfyFlags(s->ts));
+}
+
+static PyObject *rpmts_get_vfylevel(rpmtsObject *s, void *closure)
+{
+    return Py_BuildValue("i", rpmtsVfyLevel(s->ts));
+}
+
 static char rpmts_doc[] =
   "A python rpm.ts object represents an RPM transaction set.\n"
   "\n"
@@ -947,8 +975,8 @@ static char rpmts_doc[] =
   "-    rpm.RPMVSF_NORSA	if set, don't check header+payload RSA signature\n"
   "\n"
   "For convenience, there are the following masks:\n"
-  "-    rpm._RPMVSF_NODIGESTS	if set, don't check digest(s).\n"
-  "-    rpm._RPMVSF_NOSIGNATURES	if set, don't check signature(s).\n\n"
+  "-    rpm.RPMVSF_MASK_NODIGESTS	if set, don't check digest(s).\n"
+  "-    rpm.RPMVSF_MASK_NOSIGNATURES	if set, don't check signature(s).\n\n"
   "The transaction set offers an read only iterable interface for the\ntransaction elements added by the .addInstall(), .addErase() and\n.addReinstall() methods.";
 
 static PyGetSetDef rpmts_getseters[] = {
@@ -963,6 +991,8 @@ static PyGetSetDef rpmts_getseters[] = {
 	{"_prefcolor",	(getter)rpmts_get_prefcolor, (setter)rpmts_set_prefcolor, NULL},
 	{"_flags",	(getter)rpmts_get_flags, (setter)rpmts_set_flags, NULL},
 	{"_vsflags",	(getter)rpmts_get_vsflags, (setter)rpmts_set_vsflags, NULL},
+	{"_vfyflags",	(getter)rpmts_get_vfyflags, (setter)rpmts_set_vfyflags, NULL},
+	{"_vfylevel",	(getter)rpmts_get_vfylevel, (setter)rpmts_set_vfylevel, NULL},
 	{ NULL }
 };
 
