@@ -11,6 +11,14 @@ test -x $pkgconfig || {
     exit 0
 }
 
+$pkgconfig --atleast-pkgconfig-version="0.24" || {
+    cat > /dev/null
+    exit 0
+}
+
+# Under pkgconf, disables dependency resolver
+export PKG_CONFIG_MAXIMUM_TRAVERSE_DEPTH=1
+
 case $1 in
 -P|--provides)
     while read filename ; do
@@ -39,7 +47,7 @@ case $1 in
 	[ $i -eq 1 ] && echo "$pkgconfig"
 	DIR="`dirname ${filename}`"
 	export PKG_CONFIG_PATH="$DIR:$DIR/../../share/pkgconfig"
-	$pkgconfig --print-requires "$filename" 2> /dev/null | while read n r v ; do
+	$pkgconfig --print-requires --print-requires-private "$filename" 2> /dev/null | while read n r v ; do
 	    [ -n "$n" ] || continue
 	    echo -n "pkgconfig($n) "
 	    [ -n "$r" ] && [ -n "$v" ] && echo -n "$r" "$v"
